var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

import React, { Suspense } from 'react';
import PropTypes from 'prop-types';
import * as routeHelpers from './helpers';
import PageManager from './PageManager';
import LayerManager from './LayerManager';
import { compileParentPage } from "./comileParentPage";

var RouteManager = function (_React$Component) {
    _inherits(RouteManager, _React$Component);

    function RouteManager(props) {
        _classCallCheck(this, RouteManager);

        var _this = _possibleConstructorReturn(this, (RouteManager.__proto__ || Object.getPrototypeOf(RouteManager)).call(this, props));

        _this.state = {
            layers: [],
            currentPage: props.location
        };
        return _this;
    }

    _createClass(RouteManager, [{
        key: 'isCurrentLocation',
        value: function isCurrentLocation(location) {
            return this.currentPage.pathname == location.pathname;
        }
    }, {
        key: 'createLayer',
        value: function createLayer(location) {
            return {
                location: location,
                layerId: 'layer_' + routeHelpers.getRandomKey()
            };
        }
    }, {
        key: 'getLayerByLocation',
        value: function getLayerByLocation(location) {
            return this.layers.filter(function (s) {
                return s.location.pathname == location.pathname;
            })[0];
        }
    }, {
        key: 'getLastLayer',
        value: function getLastLayer() {
            return this.layers.length > 0 ? this.layers[0] : null;
        }
    }, {
        key: 'getPageRouteById',
        value: function getPageRouteById(id) {
            return this.props.routes.pageRoutes.filter(function (s) {
                return s.routeId === id;
            })[0];
        }
    }, {
        key: 'resolveLocation',
        value: function resolveLocation(location) {
            var self = this;
            this.setState({ needUpdate: false });

            var routes = this.props.routes;
            var layers = this.layers;
            var currentLayerRoute = routeHelpers.getLayerPage(routes.layerRoutes, location);

            if (currentLayerRoute) {
                // если слой это первая загружаемая страница, то устанавливаем задний фон дефолтную страницу
                if (this.isCurrentLocation(location)) {
                    var parentRouteRule = this.getPageRouteById(currentLayerRoute.parentId);
                    var basePagePath = compileParentPage({ location: location, routeRule: currentLayerRoute, parentRouteRule: parentRouteRule });
                    this.currentPage = basePagePath ? { pathname: basePagePath } : { pathname: '/' };
                }

                var locationLayer = this.getLayerByLocation(location);
                if (locationLayer) {
                    //слой с таким урл уже есть в массиве
                    var lastLayer = this.getLastLayer();
                    if (lastLayer != locationLayer) {
                        layers = layers.filter(function (s) {
                            return s != locationLayer;
                        });
                        layers.unshift(this.createLayer(location));
                        this.layers = layers;
                    } else {
                        locationLayer.location = location;
                        this.layers = layers;
                    }
                } else {
                    if (this.layers.length >= this.props.layersLimit) {
                        // удаляем самый "нижний" слой из массива
                        layers.pop();
                    }
                    layers.unshift(this.createLayer(location));
                    this.layers = layers;
                }
            } else {
                var _layers = this.layers;
                if (_layers.length > 0) {
                    _layers.forEach(function (layer) {
                        return self.closeLayer({ layerId: layer.layerId });
                    });
                }
                this.currentPage = location;
            }
        }
    }, {
        key: 'destroyLayer',
        value: function destroyLayer(_ref) {
            var layerId = _ref.layerId;

            if (!this.layers.some(function (s) {
                return s.layerId == layerId;
            })) return;
            var layers = this.layers.filter(function (s) {
                return s.layerId != layerId;
            });
            this.layers = layers;

            //если слоев не осталось, то не нужно переходить назад
            if (layers.length != 0) {
                this.props.history.replace(layers[0].location);
            } else {
                var loc = _extends({}, this.state.currentPage);
                loc.state = { returnToPage: true };
                this.props.history.replace(loc);
            }
        }
    }, {
        key: 'closeLayer',
        value: function closeLayer(_ref2) {
            var layerId = _ref2.layerId;

            this.hideLayer(layerId);
            var self = this;

            setTimeout(function () {
                return self.destroyLayer({ layerId: layerId });
            }, 400);
        }
    }, {
        key: 'hideLayer',
        value: function hideLayer(layerId) {
            var el = document.querySelector('[data-layer=' + layerId + ']');

            if (el) {
                el.classList.remove('open');
                el.classList.add('hide');
            }
        }
    }, {
        key: 'componentDidMount',
        value: function componentDidMount() {
            var self = this;
            window.addEventListener('keyup', function (e) {
                if (e.keyCode == 27) {
                    var lastLayer = self.getLastLayer();
                    lastLayer && self.closeLayer({ layerId: lastLayer.layerId });
                }
            });
        }
    }, {
        key: 'shouldComponentUpdate',
        value: function shouldComponentUpdate(props, state) {
            return state.needUpdate;
        }
    }, {
        key: 'UNSAFE_componentWillMount',
        value: function UNSAFE_componentWillMount() {
            this.resolveLocation(this.props.location);
        }
    }, {
        key: 'UNSAFE_componentWillReceiveProps',
        value: function UNSAFE_componentWillReceiveProps(_ref3) {
            var location = _ref3.location;

            this.resolveLocation(location);
        }
    }, {
        key: 'render',
        value: function render() {
            var _this2 = this;

            var _props = this.props,
                location = _props.location,
                notFound = _props.notFound,
                routes = _props.routes,
                routeWrappers = _props.routeWrappers,
                LoadingComponent = _props.loadingComponent,
                ErrorBoundary = _props.errorBoundary,
                withLayerIdContext = _props.withLayerIdContext;
            var _state = this.state,
                currentPage = _state.currentPage,
                layers = _state.layers;


            return React.createElement(
                ErrorBoundary,
                null,
                React.createElement(
                    Suspense,
                    { fallback: LoadingComponent },
                    React.createElement(PageManager, {
                        routeWrappers: routeWrappers,
                        pageLocation: currentPage,
                        location: location,
                        routes: routes,
                        notFound: notFound
                    }),
                    layers.map(function (layer) {
                        return React.createElement(LayerManager, _extends({
                            key: layer.layerId
                        }, layer, {
                            routeWrappers: routeWrappers,
                            routes: routes.layerRoutes,
                            onCloseLayer: _this2.closeLayer.bind(_this2),
                            withLayerIdContext: withLayerIdContext
                        }));
                    })
                )
            );
        }
    }, {
        key: 'currentPage',
        set: function set(currentPage) {
            this.setState({ currentPage: currentPage, needUpdate: true });
        },
        get: function get() {
            return this.state.currentPage;
        }
    }, {
        key: 'layers',
        set: function set(layers) {
            this.setState({ layers: layers, needUpdate: true });
        },
        get: function get() {
            return this.state.layers.map(function (s) {
                return s;
            });
        }
    }]);

    return RouteManager;
}(React.Component);

RouteManager.propTypes = {
    routes: PropTypes.shape({
        pageRoutes: PropTypes.array.isRequired,
        layerRoutes: PropTypes.array.isRequired
    }).isRequired,
    notFound: PropTypes.func,
    location: PropTypes.object.isRequired,
    history: PropTypes.object.isRequired,
    layersLimit: PropTypes.number,
    routeWrappers: PropTypes.arrayOf(PropTypes.func),
    withLayerIdContext: PropTypes.bool
};
RouteManager.defaultProps = {
    layersLimit: 5
};
export { RouteManager as default };