var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

import { matchPath } from "react-router";
import pathToRegexp from 'path-to-regexp';

export var compileParentPage = function compileParentPage(_ref) {
    var location = _ref.location,
        routeRule = _ref.routeRule,
        parentRouteRule = _ref.parentRouteRule;

    if (!parentRouteRule) return null;

    //Проверяем есть ли параметры в родительском роуте
    var params = getRouteParams(parentRouteRule.path) || [];

    if (params.length === 0) {
        return parentRouteRule.path;
    }

    //Достаем параметры из дочернего узла
    var paramValues = getParamsValues(location, routeRule);

    return compileUrl(parentRouteRule.path, params, paramValues);
};

/**
 * Формируем новый урл на основе правила из роутинга и схемы параметров
 * @param path
 * @param params
 * @param values
 * @return {string}
 */
var compileUrl = function compileUrl(path, params) {
    var values = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};

    var paramsObject = params.reduce(function (res, paramKey) {
        res[paramKey] = values[paramKey] || '';
        return res;
    }, {});
    var toPath = pathToRegexp.compile(path);
    return toPath(paramsObject);
};

/**
 * Достаем параметры в указанном урле
 * @param location
 * @param routeRule
 * @return {*}
 */
var getParamsValues = function getParamsValues(location, routeRule) {
    var matchResult = matchPath(location.pathname, routeRule);
    return matchResult && matchResult.params;
};

/**
 * Возвращаем параметры необходимые для указанного роута
 * @param pathname
 * @return {Array}
 */
var getRouteParams = function getRouteParams(pathname) {
    var tokens = pathToRegexp.parse(pathname);
    return (tokens || []).filter(function (token) {
        return isObject(token) && !!token.name && token.name !== 0;
    }).map(function (token) {
        return token.name;
    });
};

function isObject(value) {
    return value !== null && (typeof value === 'undefined' ? 'undefined' : _typeof(value)) === 'object';
}