var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

import pathToRegexp from 'path-to-regexp';
import React from 'react';
import { Route } from 'react-router-dom';
import LayoutRoute from './LayoutRoute';

export var isLayerPage = function isLayerPage(routes, location) {
    return getLayerPage(routes, location) != null;
};

export var getLayerPage = function getLayerPage(routes, location) {
    return routes.filter(function (s) {
        var re = pathToRegexp(s.path, []);
        return re.exec(location.pathname) != null;
    })[0];
};

export var equalLocations = function equalLocations(a, b) {
    return a.pathname == b.pathname && a.hash == b.hash && a.search == b.search;
};

export var getRandomKey = function getRandomKey() {
    return Math.floor(Math.random() * (999999999 - 100000000)) + 100000000;
};

export var generateRouteComponent = function generateRouteComponent(_ref) {
    var routeId = _ref.routeId,
        props = _ref.props,
        routeWrappers = _ref.routeWrappers;

    var key = 'page_route_' + routeId;
    if (!routeId) throw 'RouteId must be set';

    if (props.path) {
        var RouteComponent = getRouteComponent(key, props, routeWrappers);
        return React.createElement(RouteComponent, _extends({ key: key }, props));
    } else if (props.render) {
        return props.render({ key: key });
    }
};

export var getRouteComponent = function getRouteComponent(key, props, routeWrappers) {
    var layout = props.layout;

    var routeComponent = Route;

    if (layout) routeComponent = LayoutRoute(routeComponent);

    if (routeWrappers && routeWrappers.length > 0) routeComponent = routeWrappers.reduce(function (routeComponent, wrapper) {
        return wrapper(routeComponent, props);
    }, routeComponent);

    return routeComponent;
};

export var transformRoutes = function transformRoutes(routes, defaultLayout, defaultLayerLayout) {
    return transformSectionRoutes(routes, defaultLayout, defaultLayerLayout);
};

function transformSectionRoutes(routes, defaultLayout, defaultLayerLayout) {
    var parent = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : null;

    return routes.reduce(function (r, rule) {
        var route = createRoute(rule, defaultLayout, defaultLayerLayout, parent);
        if (rule.isLayer) {
            r.layerRoutes = [].concat(_toConsumableArray(r.layerRoutes), [route]);
        } else {
            r.pageRoutes = [].concat(_toConsumableArray(r.pageRoutes), [route]);
        }

        if (rule.nested) {
            var sectionRoutes = transformSectionRoutes(rule.nested, defaultLayout, defaultLayerLayout, route);
            r.layerRoutes = [].concat(_toConsumableArray(r.layerRoutes), _toConsumableArray(sectionRoutes.layerRoutes));
            r.pageRoutes = [].concat(_toConsumableArray(r.pageRoutes), _toConsumableArray(sectionRoutes.pageRoutes));
        }

        return r;
    }, { pageRoutes: [], layerRoutes: [] });
}

function createRoute(rule, defaultLayout, defaultLayerLayout, parent) {
    var cleanRule = _extends({}, rule);
    delete cleanRule.nested;
    var route = _extends({
        routeId: rule.name + '_' + getRandomKey()
    }, cleanRule, {
        parentId: parent ? parent.routeId : null
    });
    if (rule.isLayer) {
        route.layout = rule.layout === undefined ? defaultLayerLayout : rule.layout;
    } else {
        route.layout = rule.layout === undefined ? defaultLayout : rule.layout;
    }
    return route;
}